/* sane - Scanner Access Now Easy.  Copyright (C) 2001 Henning Meier-Geinitz.
   HP Scanjet modifications Copyright (C) 2001 Stefan Nilsen

   Based on documentation from Mustek and National Semiconductor.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2 of the License, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for additional
   uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files to
   produce an executable, this does not by itself cause the resulting
   executable to be covered by the GNU General Public License.  Your use of
   that executable is in no way restricted on account of linking the SANE
   library code into it.

   This exception does not, however, invalidate any other reasons why the
   executable file might be covered by the GNU General Public License.

   If you submit changes to SANE to the maintainers to be included in a
   subsequent release, you agree by submitting the changes that those changes
   may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice whether
   to permit this exception to apply to your modifications.  If you do not
   wish that, delete this exception notice.

   This file is originally a test program for the Mustek Bearpaw 1200.
   I (Stefan Nilsen) have now modified it for HP ScanJet 2200C.
   I (Phil Underwood) have modified it to work with SANE (almost) (sortof) (ish)

   Last updated: 2001-11-05
*/
enum lm9832_Option
  {
    OPT_NUM_OPTS = 0,
    OPT_MODE_GROUP,
    OPT_DEPTH,
    OPT_RESOLUTION,
    OPT_PREVIEW,

    OPT_GEOMETRY_GROUP,
    OPT_TL_X,
    OPT_TL_Y,
    OPT_BR_X,
    OPT_BR_Y,

    NUM_OPTIONS
  };

/* allow to check what make of scanner is connected */
enum lm9832_Make
{
  LM9832_EPSON1250 = 0,
  LM9832_HP2200 = 1
};

typedef struct _scanner_chars {
  const char* id; /* identifying string of scanner (to be used in config) */
  const char* vendor;
  const char* model;
  const char* type;
  
  double fspi;
  SANE_Int dpi;
  SANE_Int dpi_list[9];
  SANE_Range x_range;
  SANE_Range y_range;
  double sensor_separation; /* Y axis distance between R&G, G&B sensors in inches */
  SANE_Int y_offset; /* Y axis distance to start of scan area */
  SANE_Byte tr_pulse_length;
  SANE_Word ideal_step; /* fast stepping step_size at MCLK_DIV= 1/4 */
} scanner_chars;

typedef struct _lm9832_Device
  {
    struct _lm9832_Device *next;
    SANE_Device sane;
    SANE_Handle handle;
    SANE_Int scanner_make; 
}
lm9832_Device;

typedef struct _lm9832_Scanner
{
  /* all the state needed to define a scan request: */
  
  SANE_Option_Descriptor opt[NUM_OPTIONS];
  SANE_Word val[NUM_OPTIONS];
  
  /* Parsed option values and variables that are valid only during
     actual scanning: */
  SANE_Bool scanning;
  SANE_Parameters params;
  
  SANE_Int fd;			/* USB filedescriptor */
  SANE_Int aborted_by_user;
  unsigned long bytes_per_line; /* bytes, not including the extra statusbytes */
  unsigned int	hdpi_dividerx10;
  unsigned int  scan_mode;
  char mclk_div; 
  unsigned char rrggbb[6];
  unsigned char rgbrgb[6];
  unsigned int  rrggbb_index; /* data used in calibration */
  struct {
    unsigned int red;
    unsigned int green;
    unsigned int blue;
  } target_value_min;
  struct {
    unsigned int red;
    unsigned int green;
    unsigned int blue;
  } target_value_low;
  struct {
    unsigned int red;
    unsigned int green;
    unsigned int blue;
  } target_value_high;
  struct {
    unsigned int red;
    unsigned int green;
    unsigned int blue;
  } target_value_max;
  
  int blocking;               /* using blocking calls */
  int bytes_left;            /* number of bytes still to retrieve for image*/
  int bytes_done;            /* number of bytes already read this line*/

  unsigned long colour_buffer_lines; /* number of lines needed to buffer for the green data*/
  unsigned char *red_buffer; /* buffer for blue image data*/
  unsigned char *green_buffer; /* buffer for green image data*/
  unsigned long red_buf_count; /* location of current *pixel* in buffer (not byte)*/
  unsigned long green_buf_count;
  unsigned long lines_done;
  time_t timer;              /* time when lamp was turned on */
  /* scanner dependent/low-level state: */
  scanner_chars *details;
  lm9832_Device *hw;
}
lm9832_Scanner;
